/* This file is part of the KDE project
   Copyright (C) 2002-2003 Norbert Andres <nandres@web.de>
             (C) 2002-2003 Ariya Hidayat <ariya@kde.org>
             (C) 2002      Laurent Montel <montel@kde.org>
             (C) 1999 David Faure <faure@kde.org>

   This library is free software; you can redistribute it and/or
   modify it under the terms of the GNU Library General Public
   License as published by the Free Software Foundation; either
   version 2 of the License, or (at your option) any later version.

   This library is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   Library General Public License for more details.

   You should have received a copy of the GNU Library General Public License
   along with this library; see the file COPYING.LIB.  If not, write to
   the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
 * Boston, MA 02110-1301, USA.
*/

#ifndef CSVDIALOG_H
#define CSVDIALOG_H

#include <kdialogbase.h>

class TQVBoxLayout;
class TQHBoxLayout;
class TQGridLayout;
class TQButtonGroup;
class TQComboBox;
class TQCheckBox;
class TQLabel;
class TQLineEdit;
class TQPushButton;
class TQRadioButton;
class TQTable;


namespace KSpread
{
class View;

/**
 * Provides dialog for managing CSV (comma separated value) data.
 *
 * Currently CSVDialog is used for converting text into columns,
 * inserting text file and pasting text from clipboard, where conversion
 * from CSV (comma separated value) data is is all required. 
 * The different purposed mentioned above is determined
 * using mode, which can be Column, File, or Clipboard respectively.
 *
*/
class CSVDialog : public KDialogBase
{
  TQ_OBJECT
  

 public:

  enum Mode { Clipboard, File, Column };
  enum Header { TEXT, NUMBER, DATE, CURRENCY };

  CSVDialog( View * parent, const char * name, TQRect const & rect, Mode mode);

  ~CSVDialog();

  bool cancelled();

 protected:
  void accept();


 private:
  View* m_pView;

  TQGridLayout* MyDialogLayout;
  TQHBoxLayout* Layout1;
  TQGridLayout* m_delimiterBoxLayout;
  TQGridLayout* m_formatBoxLayout;
  TQTable* m_sheet;
  TQButtonGroup* m_delimiterBox;
  TQRadioButton* m_radioComma;
  TQRadioButton* m_radioSemicolon;
  TQRadioButton* m_radioTab;
  TQRadioButton* m_radioSpace;
  TQRadioButton* m_radioOther;
  TQLineEdit* m_delimiterEdit;
  TQButtonGroup* m_formatBox;
  TQRadioButton* m_radioNumber;
  TQRadioButton* m_radioText;
  TQRadioButton* m_radioCurrency;
  TQRadioButton* m_radioDate;
  TQComboBox* m_comboLine;
  TQComboBox* m_comboQuote;
  TQLabel* TextLabel3;
  TQLabel* TextLabel2;
  TQCheckBox * m_ignoreDuplicates;

  void fillSheet();
  void fillComboBox();
  void setText(int row, int col, const TQString& text);
  void adjustRows(int iRows);
  int  getHeader(int col);
  TQString getText(int row, int col);

  bool m_cancelled;
  int   m_adjustRows;
  int   m_startline;
  TQChar m_textquote;
  TQString m_delimiter;
  TQString m_data;
  TQByteArray m_fileArray;
  TQRect m_targetRect;
  Mode  m_mode;

 private slots:
  void returnPressed();
  void formatClicked(int id);
  void delimiterClicked(int id);
  void lineSelected(const TQString& line);
  void textquoteSelected(const TQString& mark);
  void currentCellChanged(int, int col);
  void textChanged ( const TQString & );
  void ignoreDuplicatesChanged(int);
};

} // namespace KSpread

#endif // CVSDIALOG_H
